﻿function Get-NameRegex {
    param(
        $Str,
        $Regex,
        $Exclude,
        [switch] $End
    )
    # 查詢 Regex 字串中定義的 TAG，並將其作為 Scriptblock
    $Keyfind = {
        process{
            if ($_ -match "\(\?\<(.*?)\>") {
                $Matches[1]
            } else {
                Write-Error "$_ 沒有定義(?<TAG>)"
            }
        }
    }

    # 將原函式轉為 Scriptblock，以方便 Exclude 功能遞迴呼叫
    $InnerScrip = {
        # 取用管線傳入的 hash 進行 match
        process {
            $Key = $_["Regex"] | &$Keyfind
            if ($_["Target"] -match $_["Regex"]) {
                $Result = @{ 
                    $Key = $Matches[$Key]
                    # 將錨定字串中的特殊字元轉換為一般字串
                    Anchor = $Matches[0] -replace "([\[\]\(\)])", '\$1'
                }
                $LR = $_["Target"] -split $Result["Anchor"]
                $Result.Add("L", $LR[0])
                $Result.Add("R", $LR[1])
                # 對 Anchor 結果，以 $Exclude 進行比對，符合則加入剩餘字串重新 match
                if ($Exclude -and($Result["Anchor"] -match $Exclude)) {
                    $Result.Add("Regex", $_["Regex"])
                    $Result.Add("Target", $Result["L"])
                    return $Result | &$InnerScrip
                }
                # 結尾模式，僅輸出符合 TAGKEY 的 hash
                if ($End) { return @{ $Key = $Result[$Key] } } else { return $Result }
            }
        }
    }

    #Write-Host $Str
    # Regex 可輸入複數個
    $Regex | % { @{ Target = $Str; Regex = $_; } } | &$InnerScrip
}

# Example
# $RS1 | % { Get-NameRegex $_["L"] -Regex $NameRegex -Exclude "(BD|DVD|HDTV)RIP" -End }
# $TestName | % { Get-NameRegex $_ -Regex $Regex } | % { $(Get-NameRegex $_["L"] -Regex $NameRegex -Exclude "(BD|DVD|HDTV)RIP" -End) + $_ }
# SIG # Begin signature block
# MIIFxwYJKoZIhvcNAQcCoIIFuDCCBbQCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQU2SfubhaWpugweGH+53eZ/igq
# VQOgggNSMIIDTjCCAjagAwIBAgIQV5lzhhF94bJEhqpMRjsGlzANBgkqhkiG9w0B
# AQUFADAqMRMwEQYDVQQDDApOaWNrIENoYW5nMRMwEQYDVQQKDAplYXJpY2sub3Jn
# MB4XDTIyMTAzMTE3NTEzNVoXDTIzMTAzMTE4MTEzNVowMzEiMCAGCSqGSIb3DQEJ
# ARYTcGNuaWNrMTAwQGdtYWlsLmNvbTENMAsGA1UEAwwETmljazCCASIwDQYJKoZI
# hvcNAQEBBQADggEPADCCAQoCggEBANj4xp1L4NLChJS8NthMMln7V6LccMjvtf6P
# UNBk4TaV6bGCLAtwT1AcU2X+uloOag85JqhEu6zR4yFZqNDcKq5eXVZsd3HIH4nH
# q1H5GAF3LNPhGj7DXjBdwrZTpMcmYJ81CaWbDJzjTAEd3MkCGNilpvRMVSNC7l/L
# TSojIp4N1tvIqsT2tqU8i9vSKeXZhgWVSx27HbIP54J+VpcLqB9YUPUdeLxpYdIQ
# GlCk3s/5zzpZUBLsns/OJgJmoXcModBxOIfPlchDODqTn6TVzKdszLdXfjymtx1t
# eF812HHGkZnzov3uKQV3n6zuM/lfcRxVbCuIT5UzBZX7BF/DTAkCAwEAAaNnMGUw
# DgYDVR0PAQH/BAQDAgeAMBMGA1UdJQQMMAoGCCsGAQUFBwMDMB8GA1UdIwQYMBaA
# FBXyUrhn78ppKe7GRmHS6vbsNae/MB0GA1UdDgQWBBSJSdIOoqzCt5WY6WJNZ4MO
# CQGPaDANBgkqhkiG9w0BAQUFAAOCAQEAUkJt4vMLRqdtBu+gWw/EGGvRLQYriQhv
# d1FHCK8gC1O3s5tre9y3fuLpx06fCvERRf544Q02EaKq6MtgODZVroe1EUJ5YxZR
# K0iDpW/uolGXMth1U+Pb2Y9XrFa8LuYJcq+B5QE1eb5BSLDmgN5ML02eHoh523xs
# pq7moIexPszsneLhtW+qAUrgi4zi4JCLmssLIC7OdfnA+sFcj+y9f0OtBZPeq5Oi
# XTIrnft6uVOoeSy4UCZ3arbIG/28+CJ+U8lMro+Gput/OaV5ctZmh3K/bjoXJ1oW
# TeAge6EFyw4ARhr1QyxpFFLNtuKGC3Y437L0tUzx5tGRQdyhHtm8RTGCAd8wggHb
# AgEBMD4wKjETMBEGA1UEAwwKTmljayBDaGFuZzETMBEGA1UECgwKZWFyaWNrLm9y
# ZwIQV5lzhhF94bJEhqpMRjsGlzAJBgUrDgMCGgUAoHgwGAYKKwYBBAGCNwIBDDEK
# MAigAoAAoQKAADAZBgkqhkiG9w0BCQMxDAYKKwYBBAGCNwIBBDAcBgorBgEEAYI3
# AgELMQ4wDAYKKwYBBAGCNwIBFTAjBgkqhkiG9w0BCQQxFgQUoqQp8TNHQrFdDrpM
# 6+iDq3QvqIUwDQYJKoZIhvcNAQEBBQAEggEA2OyzZtIX4tS461TSPehuuWbtFqSR
# T4n0WFLMuFQIE53GlHcSUnz0Vx0XxfNcTCnboPX9X+1EGlMBjXnnXdv6Nu1z2HiK
# QSEvNush5MCDeeiHxyxAF+VyFl7l57jr69K8wwjASb2RZ/0/kXU5UDgj1WfV2JwM
# D2dYbUXWWHJ4XgWZchUYXcNlbu96Is3RoH7JNwCuagiydTR2LK9QdW1jmeqbLAjC
# S+NptyWk3g/XynAjwDvPSVqM91TxEVDZeuKG6o9n7fQwRl/AKe8pdPDR100+JBbo
# RN6C5fxhSccUns1UvqASuggka4ZbNEqoh2w4vofb31qsT5qVXoLZ/kMJMA==
# SIG # End signature block
